@@ -163,7 +163,7 @@ int main_configure(char *arg1, char *arg2) {
 
 int drop_privileges(int critical) {
 #if defined(USE_WIN32) || defined(__vms) || defined(USE_OS2)
-    (void)critical; /* skip warning about unused parameter */
+    (void)critical; /* squash the unused parameter warning */
 #else
 #ifdef HAVE_SETGROUPS
     gid_t gr_list[1];
